@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -215,7 +215,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   bevel_width=(size_t) (frame_info->outer_bevel+frame_info->inner_bevel);
   x=(ssize_t) frame_info->width-frame_info->x-bevel_width;
   y=(ssize_t) frame_info->height-frame_info->y-bevel_width;
-  if ((x < (ssize_t) image->columns) |  (y < (ssize_t) image->rows))
+  if ((x < (ssize_t) image->columns) ||  (y < (ssize_t) image->rows))
     ThrowImageException(OptionError,"FrameIsLessThanImageSize");
   /*
     Initialize framed image attributes.
