@@ -1011,30 +1011,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 (target_colorspace == UndefinedColorspace))
               ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
                 name);
-             if (((source_colorspace == LinearGRAYColorspace) ||
-                  (source_colorspace == GRAYColorspace)) &&
-                 (SetImageGray(image,exception) == MagickFalse))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == CMYKColorspace) &&
-                 (image->colorspace != CMYKColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == XYZColorspace) &&
-                 (image->colorspace != XYZColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == YCbCrColorspace) &&
-                 (image->colorspace != YCbCrColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace != CMYKColorspace) &&
-                 (source_colorspace != LabColorspace) &&
-                 (source_colorspace != XYZColorspace) &&
-                 (source_colorspace != YCbCrColorspace) &&
-                 (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
             switch (image->rendering_intent)
             {
               case AbsoluteIntent: intent=INTENT_ABSOLUTE_COLORIMETRIC; break;
@@ -1130,20 +1106,25 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               for (x=0; x < (ssize_t) image->columns; x++)
               {
                 if (target_channels == 1)
-                  SetPixelGray(image,target_scale*QuantumRange*(*p),q);
+                  SetPixelGray(image,ClampToQuantum(target_scale*
+                    QuantumRange*(*p)),q);
                 else
-                  SetPixelRed(image,target_scale*QuantumRange*(*p),q);
+                  SetPixelRed(image,ClampToQuantum(target_scale*
+                    QuantumRange*(*p)),q);
                 p++;
                 if (target_channels > 1)
                   {
-                    SetPixelGreen(image,target_scale*QuantumRange*(*p),q);
+                    SetPixelGreen(image,ClampToQuantum(target_scale*
+                      QuantumRange*(*p)),q);
                     p++;
-                    SetPixelBlue(image,target_scale*QuantumRange*(*p),q);
+                    SetPixelBlue(image,ClampToQuantum(target_scale*
+                      QuantumRange*(*p)),q);
                     p++;
                   }
                 if (target_channels > 3)
                   {
-                    SetPixelBlack(image,target_scale*QuantumRange*(*p),q);
+                    SetPixelBlack(image,ClampToQuantum(target_scale*
+                      QuantumRange*(*p)),q);
                     p++;
                   }
                 q+=GetPixelChannels(image);
@@ -1420,8 +1401,7 @@ static void WriteTo8BimProfile(Image *image,const char *name,
     count=(ssize_t) value;
     if ((count & 0x01) != 0)
       count++;
-    if ((count < 0) || (p > (datum+length-count)) ||
-        (count > (ssize_t) length))
+    if ((count < 0) || (p > (datum+length-count)) || (count > (ssize_t) length))
       break;
     if (id != profile_id)
       p+=count;
@@ -1511,8 +1491,7 @@ static void GetProfilesFromResourceBlock(Image *image,
       break;
     p=ReadResourceLong(p,&value);
     count=(ssize_t) value;
-    if ((p > (datum+length-count)) || (count > (ssize_t) length) ||
-        (count < 0))
+    if ((p > (datum+length-count)) || (count > (ssize_t) length) || (count < 0))
       break;
     switch (id)
     {
@@ -1527,6 +1506,8 @@ static void GetProfilesFromResourceBlock(Image *image,
         /*
           Resolution.
         */
+        if (count < 10)
+          break;
         p=ReadResourceLong(p,&resolution);
         image->resolution.x=((double) resolution)/65536.0;
         p=ReadResourceShort(p,&units)+2;
@@ -1853,9 +1834,10 @@ static MagickBooleanType Sync8BimProfile(Image *image,StringInfo *profile)
       return(MagickFalse);
     id=ReadProfileMSBShort(&p,&length);
     count=(ssize_t) ReadProfileByte(&p,&length);
-    if ((count > (ssize_t) length) || (count < 0))
+    if ((count >= (ssize_t) length) || (count < 0))
       return(MagickFalse);
     p+=count;
+    length-=count;
     if ((*p & 0x01) == 0)
       (void) ReadProfileByte(&p,&length);
     count=(ssize_t) ReadProfileMSBLong(&p,&length);
