@@ -80,6 +80,7 @@ typedef struct _ResourceInfo
   MagickOffsetType
     width,
     height,
+    list_length,
     area,
     memory,
     map,
@@ -92,6 +93,7 @@ typedef struct _ResourceInfo
   MagickSizeType
     width_limit,
     height_limit,
+    list_length_limit,
     area_limit,
     memory_limit,
     map_limit,
@@ -113,6 +115,7 @@ static ResourceInfo
   {
     MagickULLConstant(0),              /* initial width */
     MagickULLConstant(0),              /* initial height */
+    MagickULLConstant(0),              /* initial list length */
     MagickULLConstant(0),              /* initial area */
     MagickULLConstant(0),              /* initial memory */
     MagickULLConstant(0),              /* initial map */
@@ -123,6 +126,7 @@ static ResourceInfo
     MagickULLConstant(0),              /* initial time */
     (INT_MAX/(5*sizeof(Quantum))),     /* width limit */
     (INT_MAX/(5*sizeof(Quantum))),     /* height limit */
+    MagickResourceInfinity,            /* list length limit */
     MagickULLConstant(3072)*1024*1024, /* area limit */
     MagickULLConstant(1536)*1024*1024, /* memory limit */
     MagickULLConstant(3072)*1024*1024, /* map limit */
@@ -207,50 +211,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
         }
       break;
     }
-    case MemoryResource:
-    {
-      if ((resource_info.memory+(MagickOffsetType) size) < 0)
-        return(MagickFalse);
-      resource_info.memory+=(MagickOffsetType) size;
-      limit=resource_info.memory_limit;
-      if ((limit == MagickResourceInfinity) ||
-          (resource_info.memory < (MagickOffsetType) limit))
-        status=MagickTrue;
-      else
-        resource_info.memory-=(MagickOffsetType) size;
-      if (logging != MagickFalse)
-        {
-          (void) FormatMagickSize(size,MagickTrue,"B",MagickFormatExtent,
-            resource_request);
-          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
-            MagickTrue,"B",MagickFormatExtent,resource_current);
-          (void) FormatMagickSize(limit,MagickTrue,"B",MagickFormatExtent,
-            resource_limit);
-        }
-      break;
-    }
-    case MapResource:
-    {
-      if ((resource_info.map+(MagickOffsetType) size) < 0)
-        return(MagickFalse);
-      resource_info.map+=(MagickOffsetType) size;
-      limit=resource_info.map_limit;
-      if ((limit == MagickResourceInfinity) ||
-          (resource_info.map < (MagickOffsetType) limit))
-        status=MagickTrue;
-      else
-        resource_info.map-=(MagickOffsetType) size;
-      if (logging != MagickFalse)
-        {
-          (void) FormatMagickSize(size,MagickTrue,"B",MagickFormatExtent,
-            resource_request);
-          (void) FormatMagickSize((MagickSizeType) resource_info.map,
-            MagickTrue,"B",MagickFormatExtent,resource_current);
-          (void) FormatMagickSize(limit,MagickTrue,"B",MagickFormatExtent,
-            resource_limit);
-        }
-      break;
-    }
     case DiskResource:
     {
       if ((resource_info.disk+(MagickOffsetType) size) < 0)
@@ -282,8 +242,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       if ((limit == MagickResourceInfinity) ||
           (resource_info.file < (MagickOffsetType) limit))
         status=MagickTrue;
-      else
-        resource_info.file-=(MagickOffsetType) size;
       if (logging != MagickFalse)
         {
           (void) FormatMagickSize(size,MagickFalse,(const char *) NULL,
@@ -313,6 +271,67 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
         }
       break;
     }
+    case MapResource:
+    {
+      if ((resource_info.map+(MagickOffsetType) size) < 0)
+        return(MagickFalse);
+      resource_info.map+=(MagickOffsetType) size;
+      limit=resource_info.map_limit;
+      if ((limit == MagickResourceInfinity) ||
+          (resource_info.map < (MagickOffsetType) limit))
+        status=MagickTrue;
+      else
+        resource_info.map-=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickTrue,"B",MagickFormatExtent,
+            resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.map,
+            MagickTrue,"B",MagickFormatExtent,resource_current);
+          (void) FormatMagickSize(limit,MagickTrue,"B",MagickFormatExtent,
+            resource_limit);
+        }
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource_info.list_length=(MagickOffsetType) size;
+      limit=resource_info.list_length_limit;
+      if ((limit == MagickResourceInfinity) || (size < limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,(const char *) NULL,
+            MagickFormatExtent,resource_request);
+          (void) FormatMagickSize(size,MagickFalse,(const char *) NULL,
+            MagickFormatExtent,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,(const char *) NULL,
+            MagickFormatExtent,resource_limit);
+        }
+      break;
+    }
+    case MemoryResource:
+    {
+      if ((resource_info.memory+(MagickOffsetType) size) < 0)
+        return(MagickFalse);
+      resource_info.memory+=(MagickOffsetType) size;
+      limit=resource_info.memory_limit;
+      if ((limit == MagickResourceInfinity) ||
+          (resource_info.memory < (MagickOffsetType) limit))
+        status=MagickTrue;
+      else
+        resource_info.memory-=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickTrue,"B",MagickFormatExtent,
+            resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+            MagickTrue,"B",MagickFormatExtent,resource_current);
+          (void) FormatMagickSize(limit,MagickTrue,"B",MagickFormatExtent,
+            resource_limit);
+        }
+      break;
+    }
     case ThreadResource:
     {
       limit=resource_info.thread_limit;
@@ -688,31 +707,11 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
-    {
-      resource=(MagickSizeType) resource_info.width;
-      break;
-    }
-    case HeightResource:
-    {
-      resource=(MagickSizeType) resource_info.height;
-      break;
-    }
     case AreaResource:
     {
       resource=(MagickSizeType) resource_info.area;
       break;
     }
-    case MemoryResource:
-    {
-      resource=(MagickSizeType) resource_info.memory;
-      break;
-    }
-    case MapResource:
-    {
-      resource=(MagickSizeType) resource_info.map;
-      break;
-    }
     case DiskResource:
     {
       resource=(MagickSizeType) resource_info.disk;
@@ -723,6 +722,26 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
       resource=(MagickSizeType) resource_info.file;
       break;
     }
+    case HeightResource:
+    {
+      resource=(MagickSizeType) resource_info.height;
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource=(MagickSizeType) resource_info.list_length;
+      break;
+    }
+    case MapResource:
+    {
+      resource=(MagickSizeType) resource_info.map;
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=(MagickSizeType) resource_info.memory;
+      break;
+    }
     case ThreadResource:
     {
       resource=(MagickSizeType) resource_info.thread;
@@ -738,6 +757,11 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
       resource=(MagickSizeType) resource_info.time;
       break;
     }
+    case WidthResource:
+    {
+      resource=(MagickSizeType) resource_info.width;
+      break;
+    }
     default:
       break;
   }
@@ -778,31 +802,11 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
-    {
-      resource=resource_info.width_limit;
-      break;
-    }
-    case HeightResource:
-    {
-      resource=resource_info.height_limit;
-      break;
-    }
     case AreaResource:
     {
       resource=resource_info.area_limit;
       break;
     }
-    case MemoryResource:
-    {
-      resource=resource_info.memory_limit;
-      break;
-    }
-    case MapResource:
-    {
-      resource=resource_info.map_limit;
-      break;
-    }
     case DiskResource:
     {
       resource=resource_info.disk_limit;
@@ -813,6 +817,26 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
       resource=resource_info.file_limit;
       break;
     }
+    case HeightResource:
+    {
+      resource=resource_info.height_limit;
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource=resource_info.list_length_limit;
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=resource_info.memory_limit;
+      break;
+    }
+    case MapResource:
+    {
+      resource=resource_info.map_limit;
+      break;
+    }
     case ThreadResource:
     {
       resource=resource_info.thread_limit;
@@ -828,6 +852,11 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
       resource=resource_info.time_limit;
       break;
     }
+    case WidthResource:
+    {
+      resource=resource_info.width_limit;
+      break;
+    }
     default:
       break;
   }
@@ -867,6 +896,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     area_limit[MagickFormatExtent],
     disk_limit[MagickFormatExtent],
     height_limit[MagickFormatExtent],
+    list_length_limit[MagickFormatExtent],
     map_limit[MagickFormatExtent],
     memory_limit[MagickFormatExtent],
     time_limit[MagickFormatExtent],
@@ -883,6 +913,8 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     MagickFormatExtent,width_limit);
   (void) FormatMagickSize(resource_info.height_limit,MagickFalse,"P",
     MagickFormatExtent,height_limit);
+  (void) FormatMagickSize(resource_info.list_length_limit,MagickFalse,"P",
+    MagickFormatExtent,list_length_limit);
   (void) FormatMagickSize(resource_info.area_limit,MagickFalse,"P",
     MagickFormatExtent,area_limit);
   (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,"B",
@@ -900,6 +932,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   (void) FormatLocaleFile(file,"Resource limits:\n");
   (void) FormatLocaleFile(file,"  Width: %s\n",width_limit);
   (void) FormatLocaleFile(file,"  Height: %s\n",height_limit);
+  (void) FormatLocaleFile(file,"  List length: %s\n",list_length_limit);
   (void) FormatLocaleFile(file,"  Area: %s\n",area_limit);
   (void) FormatLocaleFile(file,"  Memory: %s\n",memory_limit);
   (void) FormatLocaleFile(file,"  Map: %s\n",map_limit);
@@ -973,36 +1006,6 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
         }
       break;
     }
-    case MemoryResource:
-    {
-      resource_info.memory-=size;
-      assert(resource_info.memory >= 0);
-      if (logging != MagickFalse)
-        {
-          (void) FormatMagickSize(size,MagickFalse,"B",MagickFormatExtent,
-            resource_request);
-          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
-            MagickTrue,"B",MagickFormatExtent,resource_current);
-          (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,"B",
-            MagickFormatExtent,resource_limit);
-        }
-      break;
-    }
-    case MapResource:
-    {
-      resource_info.map-=size;
-      assert(resource_info.map >= 0);
-      if (logging != MagickFalse)
-        {
-          (void) FormatMagickSize(size,MagickFalse,"B",MagickFormatExtent,
-            resource_request);
-          (void) FormatMagickSize((MagickSizeType) resource_info.map,
-            MagickTrue,"B",MagickFormatExtent,resource_current);
-          (void) FormatMagickSize(resource_info.map_limit,MagickTrue,"B",
-            MagickFormatExtent,resource_limit);
-        }
-      break;
-    }
     case DiskResource:
     {
       resource_info.disk-=size;
@@ -1048,6 +1051,51 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
         }
       break;
     }
+    case ListLengthResource:
+    {
+      resource_info.list_length=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,(const char *) NULL,
+            MagickFormatExtent,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.list_length,
+            MagickFalse,(const char *) NULL,MagickFormatExtent,
+            resource_current);
+          (void) FormatMagickSize(resource_info.list_length_limit,MagickFalse,
+            (const char *) NULL,MagickFormatExtent,resource_limit);
+        }
+      break;
+    }
+    case MapResource:
+    {
+      resource_info.map-=size;
+      assert(resource_info.map >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,"B",MagickFormatExtent,
+            resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.map,
+            MagickTrue,"B",MagickFormatExtent,resource_current);
+          (void) FormatMagickSize(resource_info.map_limit,MagickTrue,"B",
+            MagickFormatExtent,resource_limit);
+        }
+      break;
+    }
+    case MemoryResource:
+    {
+      resource_info.memory-=size;
+      assert(resource_info.memory >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,"B",MagickFormatExtent,
+            resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+            MagickTrue,"B",MagickFormatExtent,resource_current);
+          (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,"B",
+            MagickFormatExtent,resource_limit);
+        }
+      break;
+    }
     case ThreadResource:
     {
       if (logging != MagickFalse)
@@ -1159,7 +1207,8 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
   UnlockSemaphoreInfo(resource_semaphore);
   (void) CopyMagickString(cache_path,path,MagickPathExtent);
   AppendImageFormat("cache",cache_path);
-  (void) ShredFile(cache_path);
+  if (access_utf8(cache_path,F_OK) == 0)
+    (void) ShredFile(cache_path);
   if (status == MagickFalse)
     status=ShredFile(path);
   return(status);
@@ -1314,6 +1363,14 @@ MagickPrivate MagickBooleanType ResourceComponentGenesis(void)
         100.0));
       limit=DestroyString(limit);
     }
+  (void) SetMagickResourceLimit(ListLengthResource,MagickResourceInfinity);
+  limit=GetEnvironmentValue("MAGICK_LIST_LENGTH_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(ListLengthResource,
+        StringToMagickSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
   return(MagickTrue);
 }
 
@@ -1390,74 +1447,82 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   value=(char *) NULL;
   switch (type)
   {
-    case WidthResource:
-    {
-      resource_info.width_limit=limit;
-      value=GetPolicyValue("resource:width");
-      if (value != (char *) NULL)
-        resource_info.width_limit=MagickMin(limit,StringToMagickSizeType(value,
-          100.0));
-      break;
-    }
-    case HeightResource:
-    {
-      resource_info.height_limit=limit;
-      value=GetPolicyValue("resource:height");
-      if (value != (char *) NULL)
-        resource_info.height_limit=MagickMin(limit,StringToMagickSizeType(
-          value,100.0));
-      break;
-    }
     case AreaResource:
     {
-      resource_info.area_limit=limit;
       value=GetPolicyValue("resource:area");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.area_limit=limit;
+      else
         resource_info.area_limit=MagickMin(limit,StringToMagickSizeType(value,
           100.0));
       break;
     }
-    case MemoryResource:
-    {
-      resource_info.memory_limit=limit;
-      value=GetPolicyValue("resource:memory");
-      if (value != (char *) NULL)
-        resource_info.memory_limit=MagickMin(limit,StringToMagickSizeType(
-          value,100.0));
-      break;
-    }
-    case MapResource:
-    {
-      resource_info.map_limit=limit;
-      value=GetPolicyValue("resource:map");
-      if (value != (char *) NULL)
-        resource_info.map_limit=MagickMin(limit,StringToMagickSizeType(value,
-          100.0));
-      break;
-    }
     case DiskResource:
     {
-      resource_info.disk_limit=limit;
       value=GetPolicyValue("resource:disk");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.disk_limit=limit;
+      else
         resource_info.disk_limit=MagickMin(limit,StringToMagickSizeType(value,
           100.0));
       break;
     }
     case FileResource:
     {
-      resource_info.file_limit=limit;
       value=GetPolicyValue("resource:file");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.file_limit=limit;
+      else
         resource_info.file_limit=MagickMin(limit,StringToMagickSizeType(value,
           100.0));
       break;
     }
+    case HeightResource:
+    {
+      value=GetPolicyValue("resource:height");
+      if (value == (char *) NULL)
+        resource_info.height_limit=limit;
+      else
+        resource_info.height_limit=MagickMin(limit,StringToMagickSizeType(
+          value,100.0));
+      break;
+    }
+    case ListLengthResource:
+    {
+      value=GetPolicyValue("resource:list-length");
+      if (value == (char *) NULL)
+        resource_info.list_length_limit=limit;
+      else
+        resource_info.list_length_limit=MagickMin(limit,
+          StringToMagickSizeType(value,100.0));
+      break;
+    }
+    case MapResource:
+    {
+      value=GetPolicyValue("resource:map");
+      if (value == (char *) NULL)
+        resource_info.map_limit=limit;
+      else
+        resource_info.map_limit=MagickMin(limit,StringToMagickSizeType(
+          value,100.0));
+      break;
+    }
+    case MemoryResource:
+    {
+      value=GetPolicyValue("resource:memory");
+      if (value == (char *) NULL)
+        resource_info.memory_limit=limit;
+      else
+        resource_info.memory_limit=MagickMin(limit,StringToMagickSizeType(
+          value,100.0));
+      break;
+    }
     case ThreadResource:
     {
-      resource_info.thread_limit=limit;
       value=GetPolicyValue("resource:thread");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.thread_limit=limit;
+      else
         resource_info.thread_limit=MagickMin(limit,StringToMagickSizeType(
           value,100.0));
       if (resource_info.thread_limit > GetOpenMPMaximumThreads())
@@ -1469,26 +1534,40 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
     }
     case ThrottleResource:
     {
-      resource_info.throttle_limit=limit;
       value=GetPolicyValue("resource:throttle");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.throttle_limit=limit;
+      else
         resource_info.throttle_limit=MagickMax(limit,StringToMagickSizeType(
           value,100.0));
       break;
     }
     case TimeResource:
     {
-      resource_info.time_limit=limit;
       value=GetPolicyValue("resource:time");
-      if (value != (char *) NULL)
+      if (value == (char *) NULL)
+        resource_info.time_limit=limit;
+      else
         resource_info.time_limit=MagickMin(limit,StringToMagickSizeType(value,
           100.0));
       ResetPixelCacheEpoch();
       break;
     }
+    case WidthResource:
+    {
+      value=GetPolicyValue("resource:width");
+      if (value == (char *) NULL)
+        resource_info.width_limit=limit;
+      else
+        resource_info.width_limit=MagickMin(limit,StringToMagickSizeType(value,
+          100.0));
+      break;
+    }
     default:
+    {
       status=MagickFalse;
       break;
+    }
   }
   if (value != (char *) NULL)
     value=DestroyString(value);
