@@ -24,7 +24,9 @@ extern "C" {
 
 extern MagickExport MagickBooleanType
   InvokeStaticImageFilter(const char *,Image **,const int,const char **,
-    ExceptionInfo *);
+    ExceptionInfo *),
+  RegisterStaticModule(const char *,ExceptionInfo *exception),
+  UnregisterStaticModule(const char *);
 
 extern ModuleExport size_t
   RegisterAAIImage(void),
@@ -355,6 +357,10 @@ extern ModuleExport void
   UnregisterYCBCRImage(void),
   UnregisterYUVImage(void);
 
+extern MagickExport void
+  RegisterStaticModules(void),
+  UnregisterStaticModules(void);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
