@@ -167,7 +167,8 @@ MagickExport char *AcquireString(const char *source)
 %    o length: the string length.
 %
 */
-MagickExport StringInfo *AcquireStringInfo(const size_t length)
+
+static StringInfo *AcquireStringInfoContainer()
 {
   StringInfo
     *string_info;
@@ -175,13 +176,23 @@ MagickExport StringInfo *AcquireStringInfo(const size_t length)
   string_info=(StringInfo *) AcquireCriticalMemory(sizeof(*string_info));
   (void) ResetMagickMemory(string_info,0,sizeof(*string_info));
   string_info->signature=MagickCoreSignature;
+  return(string_info);
+}
+
+MagickExport StringInfo *AcquireStringInfo(const size_t length)
+{
+  StringInfo
+    *string_info;
+
+  string_info=AcquireStringInfoContainer();
   string_info->length=length;
-  string_info->datum=(unsigned char *) NULL;
   if (~string_info->length >= (MagickPathExtent-1))
     string_info->datum=(unsigned char *) AcquireQuantumMemory(
       string_info->length+MagickPathExtent,sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memset(string_info->datum,0,(length+MagickPathExtent)*
+    sizeof(*string_info->datum));
   return(string_info);
 }
 
@@ -217,14 +228,10 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
 
   if (~length < MagickPathExtent)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  string_info=AcquireStringInfo(0);
+  string_info=AcquireStringInfoContainer();
   string_info->length=length;
-  if (string_info->datum == (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
-      MagickPathExtent,sizeof(*string_info->datum));
-  else
-    string_info->datum=(unsigned char *) ResizeQuantumMemory(string_info->datum,
-      length+MagickPathExtent,sizeof(*string_info->datum));
+  string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+    MagickPathExtent,sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
     {
       string_info=DestroyStringInfo(string_info);
@@ -232,6 +239,8 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
     }
   if (blob != (const void *) NULL)
     (void) memcpy(string_info->datum,blob,length);
+  (void) memset(string_info->datum+length,0,MagickPathExtent*
+    sizeof(*string_info->datum));
   return(string_info);
 }
 
@@ -247,7 +256,7 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  CloneString() replaces or frees the destination string to make it
-%  a clone of the input string plus MagickPathExtent more space so the string 
+%  a clone of the input string plus MagickPathExtent more space so the string
 %  may be worked on.
 %
 %  If source is a NULL pointer the destination string will be freed and set to
@@ -648,12 +657,9 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
     }
   string[length]='\0';
   file=close(file)-1;
-  string_info=AcquireStringInfo(0);
+  string_info=AcquireStringInfoContainer();
   string_info->path=ConstantString(filename);
   string_info->length=length;
-  if (string_info->datum != (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) RelinquishMagickMemory(
-      string_info->datum);
   string_info->datum=(unsigned char *) string;
   return(string_info);
 }
@@ -841,6 +847,8 @@ MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
   if (string_info->datum != (unsigned char *) NULL)
     string_info->datum=(unsigned char *) RelinquishMagickMemory(
       string_info->datum);
+  if (string_info->name != (char *) NULL)
+    string_info->name=DestroyString(string_info->name);
   if (string_info->path != (char *) NULL)
     string_info->path=DestroyString(string_info->path);
   string_info->signature=(~MagickCoreSignature);
@@ -1030,11 +1038,8 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
   assert(filename != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
-  string_info=AcquireStringInfo(0);
+  string_info=AcquireStringInfoContainer();
   string_info->path=ConstantString(filename);
-  if (string_info->datum != (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) RelinquishMagickMemory(
-      string_info->datum);
   string_info->datum=(unsigned char *) FileToBlob(filename,extent,
     &string_info->length,exception);
   if (string_info->datum == (unsigned char *) NULL)
@@ -1321,6 +1326,35 @@ MagickExport size_t GetStringInfoLength(const StringInfo *string_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t S t r i n g I n f o N a m e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetStringInfoName() returns the name associated with the string.
+%
+%  The format of the GetStringInfoName method is:
+%
+%      const char *GetStringInfoName(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+*/
+MagickExport const char *GetStringInfoName(const StringInfo *string_info)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickCoreSignature);
+  return(string_info->name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t S t r i n g I n f o P a t h                                         %
 %                                                                             %
 %                                                                             %
@@ -1565,7 +1599,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
       break;
     p++;
   }
-  (void) FormatLocaleFile(file,"%s(%.20g): ",id,(double) string_info->length);
+  (void) FormatLocaleFile(file,"%s(%.20g):\n",id,(double) string_info->length);
   if (i == string_info->length)
     {
       for (i=0; i < string_info->length; i++)
@@ -1667,15 +1701,15 @@ MagickExport char *SanitizeString(const char *source)
 
   const char
     *q;
-  
+
   register char
     *p;
-  
+
   static char
     whitelist[] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
       "$-_.+!*'(),{}|\\^~[]`\"><#%;/?:@&=";
-  
+
   sanitize_source=AcquireString(source);
   p=sanitize_source;
   q=sanitize_source+strlen(sanitize_source);
@@ -1806,7 +1840,39 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S e t S t r i n g I n f o D a t u m                                       %
+%   S e t S t r i n g I n f o N a m e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetStringInfoName() sets the name associated with the string.
+%
+%  The format of the SetStringInfoName method is:
+%
+%      void SetStringInfoName(StringInfo *string_info,const char *name)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+%    o name: the name.
+%
+*/
+MagickExport void SetStringInfoName(StringInfo *string_info,const char *name)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickCoreSignature);
+  assert(name != (const char *) NULL);
+  string_info->name=ConstantString(name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t S t r i n g I n f o P a t h                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
