@@ -16,13 +16,13 @@
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -96,7 +96,7 @@ MagickExport MagickBooleanType CreateMagickThreadKey(MagickThreadKey *key,
       **keys;
 
     keys=(MagickThreadValue **) key;
-    *keys=(MagickThreadValue *) AcquireQuantumMemory(1,sizeof(*keys));
+    *keys=(MagickThreadValue *) AcquireQuantumMemory(1,sizeof(**keys));
     if (*keys != (MagickThreadValue *) NULL)
       {
         (*keys)->number_threads=GetOpenMPMaximumThreads();
