@@ -596,7 +596,6 @@ MagickExport XMLTreeInfo *DestroyXMLTree(XMLTreeInfo *xml_info)
 %    o extent: Maximum length of the string.
 %
 */
-
 MagickPrivate char *FileToXML(const char *filename,const size_t extent)
 {
   char
@@ -846,8 +845,8 @@ MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
 %    o attributes: the attribute splay-tree.
 %
 */
-MagickPrivate MagickBooleanType GetXMLTreeAttributes(const XMLTreeInfo *xml_info,
-  SplayTreeInfo *attributes)
+MagickPrivate MagickBooleanType GetXMLTreeAttributes(
+  const XMLTreeInfo *xml_info,SplayTreeInfo *attributes)
 {
   register ssize_t
     i;
@@ -1001,7 +1000,8 @@ MagickPrivate XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
 %    o path: the path (e.g. property/elapsed-time).
 %
 */
-MagickPrivate XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
+MagickPrivate XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,
+  const char *path)
 {
   char
     **components,
@@ -1484,23 +1484,26 @@ static char *ParseEntities(char *xml,char **entities,int state)
                     offset=(ssize_t) (xml-p);
                     extent=(size_t) (offset+length+strlen(entity));
                     if (p != q)
-                      p=(char *) ResizeQuantumMemory(p,extent+1,sizeof(*p));
+                      p=(char *) ResizeQuantumMemory(p,extent,sizeof(*p));
                     else
                       {
                         char
                           *extent_xml;
 
-                        extent_xml=(char *) AcquireQuantumMemory(extent+1,
+                        extent_xml=(char *) AcquireQuantumMemory(extent,
                           sizeof(*extent_xml));
                         if (extent_xml != (char *) NULL)
-                          (void) CopyMagickString(extent_xml,p,extent*
-                            sizeof(*extent_xml));
+                          {
+                            ResetMagickMemory(extent_xml,0,extent*
+                              sizeof(*extent_xml));
+                            (void) CopyMagickString(extent_xml,p,extent*
+                              sizeof(*extent_xml));
+                          }
                         p=extent_xml;
                       }
                     if (p == (char *) NULL)
                       ThrowFatalException(ResourceLimitFatalError,
                         "MemoryAllocationFailed");
-                    p[extent]='\0';
                     xml=p+offset;
                     entity=strchr(xml,';');
                   }
@@ -1531,6 +1534,8 @@ static char *ParseEntities(char *xml,char **entities,int state)
           (void) CopyMagickMemory(xml,xml+i,strlen(xml+i)+1);
         while ((*xml != '\0') && (*xml != ' '))
           xml++;
+        if (*xml == '\0')
+          break;
       }
       xml--;
       if ((xml >= p) && (*xml == ' '))
@@ -1578,7 +1583,8 @@ static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,
   return((XMLTreeInfo *) NULL);
 }
 
-static MagickBooleanType ValidateEntities(char *tag,char *xml,char **entities)
+static MagickBooleanType ValidateEntities(char *tag,char *xml,
+  const size_t depth,char **entities)
 {
   register ssize_t
     i;
@@ -1586,6 +1592,8 @@ static MagickBooleanType ValidateEntities(char *tag,char *xml,char **entities)
   /*
     Check for circular entity references.
   */
+  if (depth > MagickMaxRecursionDepth)
+    return(MagickFalse);
   for ( ; ; xml++)
   {
     while ((*xml != '\0') && (*xml != '&'))
@@ -1599,7 +1607,7 @@ static MagickBooleanType ValidateEntities(char *tag,char *xml,char **entities)
            (strncmp(entities[i],xml+1,strlen(entities[i])) == 0))
       i+=2;
     if ((entities[i] != (char *) NULL) &&
-        (ValidateEntities(tag,entities[i+1],entities) == 0))
+        (ValidateEntities(tag,entities[i+1],depth+1,entities) == 0))
       return(MagickFalse);
   }
 }
@@ -1708,11 +1716,13 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
       xml++;
     if (*xml == '\0')
       break;
-    if (strncmp(xml,"<!ENTITY",8) == 0)
+    if ((strlen(xml) > 9) && (strncmp(xml,"<!ENTITY",8) == 0))
       {
         /*
           Parse entity definitions.
         */
+        if (strspn(xml+8,XMLWhitespace) == 0)
+          break;
         xml+=strspn(xml+8,XMLWhitespace)+8;
         c=xml;
         n=xml+strspn(xml,XMLWhitespace "%");
@@ -1749,7 +1759,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
           }
         entities[i+1]=ParseEntities(v,predefined_entitites,'%');
         entities[i+2]=(char *) NULL;
-        if (ValidateEntities(n,entities[i+1],entities) != MagickFalse)
+        if (ValidateEntities(n,entities[i+1],0,entities) != MagickFalse)
           entities[i]=n;
         else
           {
