@@ -149,6 +149,8 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     else
       property=InterpretImageProperties((ImageInfo *) image_info,image,option,
         exception);
+  if (property == (char *) NULL)
+    return(DestroyImageList(image));
   (void) SetImageProperty(image,"caption",property,exception);
   property=DestroyString(property);
   caption=ConstantString(GetImageProperty(image,"caption",exception));
@@ -194,11 +196,13 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     { 
+      caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
       return(DestroyImageList(image));
     }
   if (SetImageBackgroundColor(image,exception) == MagickFalse)
     {
+      caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
       image=DestroyImageList(image);
       return((Image *) NULL);
