@@ -126,6 +126,8 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
   (void) ResetImagePage(image,"0x0+0+0");
   property=InterpretImageProperties((ImageInfo *) image_info,image,
     image_info->filename,exception);
+  if (property == (char *) NULL)
+    return(DestroyImageList(image));
   (void) SetImageProperty(image,"label",property,exception);
   property=DestroyString(property);
   label=GetImageProperty(image,"label",exception);
