@@ -330,6 +330,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine if this a PDB image file.
   */
+  (void) ResetMagickMemory(&pdb_info,0,sizeof(pdb_info));
   count=ReadBlob(image,sizeof(pdb_info.name),(unsigned char *) pdb_info.name);
   if (count != sizeof(pdb_info.name))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -594,8 +595,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             *p='\0';
             length<<=1;
             length+=MagickPathExtent;
-            comment=(char *) ResizeQuantumMemory(comment,length+MagickPathExtent,
-              sizeof(*comment));
+            comment=(char *) ResizeQuantumMemory(comment,length+
+              MagickPathExtent,sizeof(*comment));
             if (comment == (char *) NULL)
               break;
             p=comment+strlen(comment);
