@@ -459,6 +459,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       return((unsigned char *) NULL);
     }
+  (void) ResetMagickMemory(scanline,0,2*row_bytes*sizeof(*scanline));
   if (bytes_per_line < 8)
     {
       /*
@@ -927,6 +928,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     return(DestroyImageList(image));
+  image->alpha_trait=BlendPixelTrait;
   (void) SetImageBackgroundColor(image,exception);
   /*
     Interpret PICT opcodes.
@@ -1145,6 +1147,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (tile_image->alpha_trait != UndefinedPixelTrait)
                   image->alpha_trait=tile_image->alpha_trait;
               }
+            (void) SetImageBackgroundColor(tile_image,exception);
             if ((code != 0x9a) && (code != 0x9b))
               {
                 /*
