@@ -568,7 +568,7 @@ static int WebPEncodeWriter(const unsigned char *stream,size_t length,
     *image;
 
   image=(Image *) picture->custom_ptr;
-  return(length != 0 ? (WriteBlob(image,length,stream) == length) : 1);
+  return(length != 0 ? (WriteBlob(image,length,stream) == (ssize_t) length) : 1);
 }
 #endif
 
